package be.iminds.ilabt.jfed.ui.javafx.am_list_gui;

import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.Fed4FireAuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.authority.UtahClearingHouseAuthorityList;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.ui.javafx.scanner.ScanAuthorityDialogController;
import be.iminds.ilabt.jfed.ui.javafx.scanner.ScanAuthorityDialogFactory;
import be.iminds.ilabt.jfed.ui.javafx.util.EditableStringListPanel;
import be.iminds.ilabt.jfed.ui.javafx.util.ObjectPropertyBindHelper;
import be.iminds.ilabt.jfed.ui.javafx.util.SelectedObjectPropertyBinder;
import com.cathive.fx.guice.FXMLComponent;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.Property;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.stage.Stage;
import org.controlsfx.dialog.CommandLinksDialog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Optional;

/**
 * AMList: Overview of known Aggregate Managers. Allows editing, adding and retrieving from internet lists
 */
@FXMLComponent
public class AMList extends BorderPane {
    private static Logger LOG = LoggerFactory.getLogger(AMList.class);

    private ObjectProperty<AuthorityInfo> selectedAuthorityInfo = new SimpleObjectProperty<>(null);
    @FXML
    private ListView<AuthorityInfo> internalAuthorityList;
    @FXML
    private ListView<AuthorityInfo> externalAuthorityList;
    @FXML
    private HBox urlsPanelHBox;
    private UrlsPanel urlsPanel;
    @FXML
    private Button saveButton;
    @FXML
    private Button discardButton;
    @FXML
    private Button deleteButton;
    @FXML
    private Button showXmlButton;
    @FXML
    private HBox allowedCertAliasesPanelHBox;
    @FXML
    private EditableStringListPanel allowedCertAliasesPanel;
    @FXML
    private TextField nameField;
    @FXML
    private TextField urnField;
    @FXML
    private TextField urnPartField;
    @FXML
    private CheckBox reconnectCheckBox;
    @FXML
    private TextArea certField;

    private final AuthorityList authorityList;
    private final ScanAuthorityDialogFactory scanAuthorityDialogFactory;
    private final AddAuthorityDialogFactory addAuthorityDialogFactory;

    @Inject
    AMList(AuthorityList authorityList, ScanAuthorityDialogFactory scanAuthorityDialogFactory,
           AddAuthorityDialogFactory addAuthorityDialogFactory) {
        this.authorityList = authorityList;
        this.scanAuthorityDialogFactory = scanAuthorityDialogFactory;
        this.addAuthorityDialogFactory = addAuthorityDialogFactory;
    }

    public void show() {
        Stage authorityListStage = new Stage();
        authorityListStage.setTitle("View & Edit Authority List");
        authorityListStage.setScene(new Scene(this));
        authorityListStage.sizeToScene();

        authorityListStage.show();
    }

    @FXML
    private void initialize() {
        internalAuthorityList.setCellFactory(authorityInfoListView -> new InternalAuthorityCell(true, true));
        externalAuthorityList.setCellFactory(authorityInfoListView -> new ExternalAuthorityCell());

        internalAuthorityList.setItems(authorityList.authorityInfosProperty());

        urlsPanel = new UrlsPanel();
        urlsPanelHBox.getChildren().add(urlsPanel);
        HBox.setHgrow(urlsPanel, Priority.ALWAYS);

        SelectedObjectPropertyBinder<AuthorityInfo> selectedObjectPropertyBinder =
                new SelectedObjectPropertyBinder<>(true/*bidirectional*/);

        selectedAuthorityInfo.addListener((observableValue, oldAuthorityInfo, newAuthorityInfo) -> {
            if (oldAuthorityInfo != null) {
                urnPartField.textProperty().unbind();
            }

            if (newAuthorityInfo != null) {
                urlsPanel.authorityUrlsProperty().set(newAuthorityInfo.getUrls());
                allowedCertAliasesPanel.listProperty().set(newAuthorityInfo.getAllowedCertificateHostnameAliases());
                urnPartField.textProperty().bind(newAuthorityInfo.urnPartProperty());
            }
        });

        selectedObjectPropertyBinder.getBinders().add(new ObjectPropertyBindHelper<AuthorityInfo>(nameField.textProperty()) {
            @Override
            public Property objectProperty(AuthorityInfo o) {
                return o.nameProperty();
            }
        });

        selectedObjectPropertyBinder.getBinders().add(new ObjectPropertyBindHelper<AuthorityInfo>(urnField.textProperty()) {
            @Override
            public Property objectProperty(AuthorityInfo o) {
                return o.urnProperty();
            }
        });

        selectedObjectPropertyBinder.getBinders().add(new ObjectPropertyBindHelper<AuthorityInfo>(certField.textProperty()) {
            @Override
            public Property objectProperty(AuthorityInfo o) {
                return o.pemSslTrustCertProperty();
            }
        });

        selectedObjectPropertyBinder.getBinders().add(new ObjectPropertyBindHelper<AuthorityInfo>(reconnectCheckBox.selectedProperty()) {
            @Override
            public Property objectProperty(AuthorityInfo o) {
                return o.reconnectEachTimeProperty();
            }
        });

        selectedAuthorityInfo.bind(internalAuthorityList.getSelectionModel().selectedItemProperty());
        selectedObjectPropertyBinder.setSelectedObjectProperty(selectedAuthorityInfo);

        selectedAuthorityInfo.addListener((observableValue, oldAuthorityInfo, newAuthorityInfo) -> {
            boolean showButtons = newAuthorityInfo != null;
            if (saveButton != null)
                saveButton.setVisible(showButtons); //commit button is currently disabled
            discardButton.setVisible(showButtons);
            deleteButton.setVisible(showButtons);
            showXmlButton.setVisible(newAuthorityInfo != null);

            if (newAuthorityInfo == null) {
                if (oldAuthorityInfo != null) {
                    if (saveButton != null)
                        saveButton.disableProperty().unbind();
                    discardButton.disableProperty().unbind();
                }
            } else {
                if (oldAuthorityInfo != null) {
                    if (saveButton != null)
                        saveButton.disableProperty().unbind();
                    discardButton.disableProperty().unbind();
                }
                if (saveButton != null)
                    saveButton.disableProperty().bind(newAuthorityInfo.committedProperty());
                discardButton.disableProperty().bind(newAuthorityInfo.committedProperty());
            }
        });

        //clear selection when selecting in external list
        externalAuthorityList.getSelectionModel().selectedItemProperty().addListener((observableValue, oldAuthorityInfo, newAuthorityInfo) -> {
            if (newAuthorityInfo != null)
                select(null);
        });
    }

    public void select(AuthorityInfo authorityInfo) {
        if (authorityInfo == null)
            internalAuthorityList.getSelectionModel().clearSelection();
        else
            internalAuthorityList.getSelectionModel().select(authorityInfo);
    }

    @FXML
    public void saveToInternal() {
        selectedAuthorityInfo.get().commit();
    }

    @FXML
    public void discard() {
        selectedAuthorityInfo.get().restore();
    }

    @FXML
    public void delete() {
        selectedAuthorityInfo.get().delete();
    }

    @FXML
    public void viewXML() {
        if (selectedAuthorityInfo.get() == null) {
            JFDialogs.create()
                    .owner(internalAuthorityList)
                    .message("Please select an authority for which you want to see the XML string")
                    .showWarning();
            return;
        }

        AuthorityInfo authority = selectedAuthorityInfo.get();

        JFDialogs.create().owner(internalAuthorityList)
                .title(authority.getName())
                .masthead(authority.getName())
                .message(authority.getSfaAuthority().toXmlString())
                .allowCopyMessage()
                .showInformation();
    }

    @FXML
    public void addNew() {
        String newAuthUrn = addAuthorityDialogFactory.showAddAuthorityDialog();
        if (newAuthUrn == null) return;
        if (authorityList.getByUrnExact(newAuthUrn) != null) {
            JFDialogs.create()
                    .owner(internalAuthorityList)
                    .message(String.format("An authority with the URN \"%s\" already exists.\n\n" +
                            "Instead of creating a new one, the existing one will be opened for editing.", newAuthUrn))
                    .showInformation();
            select(authorityList.getByUrnExact(newAuthUrn));
        } else {
            AuthorityInfo newAuthorityInfo = authorityList.add(newAuthUrn);
            select(newAuthorityInfo);
        }
    }

    public void addToInternal(AuthorityInfo externalAuthorityInfo) {
        if (externalAuthorityInfo == null) return;
        if (authorityList.getByUrnExact(externalAuthorityInfo.getSfaAuthority().getUrnString()) != null) {
            SfaAuthority external = externalAuthorityInfo.getSfaAuthority();

            CommandLinksDialog.CommandLinksButtonType overwriteExternalDataButtonType =
                    new CommandLinksDialog.CommandLinksButtonType("Overwrite with external data",
                            "Open existing one and overwrite internal data with external data (noting saved until you click save).",
                            true);


            CommandLinksDialog.CommandLinksButtonType editInternalVersionButtonType =
                    new CommandLinksDialog.CommandLinksButtonType("Open internal version for editing",
                            "Just open the existing internal authority, and discard the external data.",
                            false);

            CommandLinksDialog dlg =
                    new CommandLinksDialog(overwriteExternalDataButtonType, editInternalVersionButtonType);
            dlg.setContentText("An authority with the URN \"" + external.getUrnString() + "\" already exists.\n\n" +
                    "What do you want to do?");
            dlg.initOwner(internalAuthorityList.getScene().getWindow());
            Optional<ButtonType> response = dlg.showAndWait();

            boolean overwrite = response.isPresent() &&
                    response.get() == overwriteExternalDataButtonType.getButtonType();

            AuthorityInfo existingAuthorityInfo =
                    authorityList.getByUrnExact(external.getUrnString());

            if (overwrite) {
                existingAuthorityInfo.getSfaAuthority().updateAll(external.getHrn(), external.getUrls(), external.getGid(), external.getType());
                existingAuthorityInfo.getSfaAuthority().setAllowedCertificateHostnameAlias(external.getAllowedCertificateHostnameAliases());
                existingAuthorityInfo.getSfaAuthority().clearPemSslTrustCerts();
                existingAuthorityInfo.getSfaAuthority().addPemSslTrustCert(external.getPemSslTrustCerts());
                existingAuthorityInfo.restore();
                select(existingAuthorityInfo);
            } else
                select(existingAuthorityInfo);
        } else {
            //store a copy in internal list
            authorityList.add(new SfaAuthority(externalAuthorityInfo.getSfaAuthority()));
        }

        externalAuthorityList.getSelectionModel().clearSelection();
    }

    public void scan() {
        SfaAuthority createdSfaAuth = scanAuthorityDialogFactory.showScanAuthorityDialog(ScanAuthorityDialogController.UserChoose.SHOW_OPEN_DISCARD, true/*dialog*/);
        if (createdSfaAuth == null) return;
        if (authorityList.getByUrnExact(createdSfaAuth.getUrnString()) != null) {

            AuthorityInfo existingAuthorityInfo =
                    authorityList.getByUrnExact(createdSfaAuth.getUrnString());


            CommandLinksDialog.CommandLinksButtonType actionA =
                    new CommandLinksDialog.CommandLinksButtonType("Overwrite with scanned data", "Open existing one and overwrite data with scanned data (noting saved until you click save).", true);
            CommandLinksDialog.CommandLinksButtonType actionB =
                    new CommandLinksDialog.CommandLinksButtonType("Discard scanned data", "Just open the existing one, and discard the scanned data.", false);
            CommandLinksDialog dlg = new CommandLinksDialog(actionA, actionB);
            dlg.initOwner(internalAuthorityList.getScene().getWindow());
            dlg.setContentText("An authority with the URN \"" + createdSfaAuth.getUrnString() + "\" already exists.\n\n" +
                    "What do you want to do?");

            Optional<ButtonType> response = dlg.showAndWait();
            boolean overwrite = response.isPresent() && response.get() == actionA.getButtonType();

            if (overwrite) {
                existingAuthorityInfo.getSfaAuthority().updateAll(createdSfaAuth.getHrn(), createdSfaAuth.getUrls(), createdSfaAuth.getGid(), createdSfaAuth.getType());
                existingAuthorityInfo.getSfaAuthority().setAllowedCertificateHostnameAlias(createdSfaAuth.getAllowedCertificateHostnameAliases());
                existingAuthorityInfo.getSfaAuthority().clearPemSslTrustCerts();
                existingAuthorityInfo.getSfaAuthority().addPemSslTrustCert(createdSfaAuth.getPemSslTrustCerts());
                existingAuthorityInfo.restore();
                select(existingAuthorityInfo);
            } else
                select(existingAuthorityInfo);
        } else {
            AuthorityInfo newAuthorityInfo = authorityList.add(createdSfaAuth);
            select(newAuthorityInfo);
        }
    }

//    @FXML
//    public void loadList() {
//        //reload from file
//        authorityList.load();
//    }

    @FXML
    public void saveList() {
        authorityList.save();
    }

    @FXML
    public void resetList() {
        authorityList.resetToInternalDefaults();
    }

    @FXML
    public void viewXMLList() {
        try {
            //write to file
            StringWriter writer = new StringWriter();
            be.iminds.ilabt.jfed.lowlevel.authority.binding.Authorities xmlAuthorities =
                    new be.iminds.ilabt.jfed.lowlevel.authority.binding.Authorities();
            for (SfaAuthority auth : authorityList.getAuthorityListModel().getAuthorities()) {
                be.iminds.ilabt.jfed.lowlevel.authority.binding.Authorities.Authority xmlAuth = auth.toXml();
                xmlAuthorities.getAuthority().add(xmlAuth);
            }

            JAXBContext context =
                    JAXBContext.newInstance(be.iminds.ilabt.jfed.lowlevel.authority.binding.Authorities.class);
            Marshaller m = context.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            m.marshal(xmlAuthorities, writer);

            writer.close();
            JFDialogs.create().owner(internalAuthorityList).message("" + writer.getBuffer().toString()).allowCopyMessage().showInformation();
        } catch (JAXBException | IOException e) {
            LOG.error("Error while writing XML List",e);
        }
    }

    @FXML
    public void loadF4f() {
        ObservableList<AuthorityInfo> externalList = FXCollections.observableArrayList();

        AuthorityListModel authorityListModel = new AuthorityListModel();
        try {
            Fed4FireAuthorityList.load(authorityListModel);
        } catch (Fed4FireAuthorityList.OfflineException e) {
            LOG.error("No internet connectivity");
        }
        int i = 0;
        for (SfaAuthority sfaAuthority : authorityListModel.getAuthorities()) {
            AuthorityInfo authorityInfo = new AuthorityInfo(null, sfaAuthority, i++);
            externalList.add(authorityInfo);
        }

        externalAuthorityList.setItems(externalList);
    }

    @FXML
    public void loadUtahHttp() {
        ObservableList<AuthorityInfo> externalList = FXCollections.observableArrayList();

        AuthorityListModel authorityListModel = new AuthorityListModel();
        UtahClearingHouseAuthorityList.load(authorityListModel);
        int i = 0;
        for (SfaAuthority sfaAuthority : authorityListModel.getAuthorities()) {
            AuthorityInfo authorityInfo = new AuthorityInfo(null, sfaAuthority, i++);
            externalList.add(authorityInfo);
        }

        externalAuthorityList.setItems(externalList);
    }

    class InternalAuthorityCell extends ListCell<AuthorityInfo> {
        private HBox hbox = new HBox();
        private HBox hbox2 = new HBox();
        private Label label = new Label();
        private Label unsavedIco = new Label();
        private Button delete = new Button("Delete");
        private Button edit = new Button("Edit");

        private boolean deletable;
        private boolean editable;

        public InternalAuthorityCell(boolean deletable, boolean editable) {
            this.deletable = deletable;
            this.editable = editable;

            delete.getStyleClass().add("deleteIco");
            edit.getStyleClass().add("editIco");

            unsavedIco.getStyleClass().add("unsavedIco");

            hbox.getChildren().add(label);
            hbox.getChildren().add(unsavedIco);
            unsavedIco.setVisible(false);
            if (deletable) {
                hbox2.getChildren().add(delete);
            }
//            hbox2.getChildren().add(edit);
            hbox2.setAlignment(Pos.CENTER_RIGHT);
            HBox.setHgrow(hbox, Priority.ALWAYS);
            HBox.setHgrow(hbox2, Priority.ALWAYS);
            hbox.getChildren().add(hbox2);
            if (!editable) edit.setText("view");
        }

        @Override
        public void updateItem(final AuthorityInfo item, boolean empty) {
            super.updateItem(item, empty);

            unsavedIco.visibleProperty().unbind();
            label.textProperty().unbind();

            if (item == null)
                setGraphic(null);
            else {
                label.textProperty().bind(item.nameProperty());
                unsavedIco.visibleProperty().bind(item.savedProperty().not());

                if (deletable)
                    delete.setOnAction(actionEvent -> {
                        item.delete();
                        select(null);
                    });

                edit.setOnAction(actionEvent -> select(item));

                setGraphic(hbox);
            }
        }
    }

    class ExternalAuthorityCell extends ListCell<AuthorityInfo> {
        private HBox hbox = new HBox();
        private HBox hbox2 = new HBox();
        private Label label = new Label();
        private Button add = new Button("Add");

        public ExternalAuthorityCell() {
            hbox.getChildren().add(label);
            hbox2.getChildren().add(add);
            hbox2.setAlignment(Pos.CENTER_RIGHT);
            HBox.setHgrow(hbox, Priority.ALWAYS);
            HBox.setHgrow(hbox2, Priority.ALWAYS);
            hbox.getChildren().add(hbox2);
            add.getStyleClass().add("addIco");
        }

        @Override
        public void updateItem(final AuthorityInfo item, boolean empty) {
            super.updateItem(item, empty);

            if (item == null)
                setGraphic(null);
            else {
                label.setText(item.getSfaAuthority().getName());

                add.setOnAction(actionEvent -> addToInternal(item));

                setGraphic(hbox);
            }
        }
    }
}
