package be.iminds.ilabt.jfed.experimenter_gui.editor.properties;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.CMIManagerFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.map.MapDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.experimenter_gui.util.BonfireLicenseUtil;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.FXUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.StageUtils;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import javafx.scene.Scene;
import javafx.stage.Stage;
import javafx.stage.Window;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 12/2/2014
 * Time: 14:45
 */
@Singleton
public class AdaptivePropertiesDialogFactory {

    private final JFedConfiguration config;
    private final AuthorityList authorityList;
    private final CMIManagerFactory cmiManagerFactory;
    private final MapDialogFactory mapDialogFactory;
    private final HighLevelTaskFactory hltf;
    private final TasksFactory tasksFactory;
    private final TaskService taskService;
    private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    private final BonfireLicenseUtil bonfireLicenseUtil;

    @Inject
    public AdaptivePropertiesDialogFactory(JFedConfiguration config, AuthorityList authorityList,
                                           CMIManagerFactory cmiManagerFactory, MapDialogFactory mapDialogFactory,
                                           HighLevelTaskFactory hltf, TasksFactory tasksFactory, TaskService taskService,
                                           AggregateManagerWrapperFactory aggregateManagerWrapperFactory, BonfireLicenseUtil bonfireLicenseUtil) {
        this.config = config;
        this.authorityList = authorityList;
        this.cmiManagerFactory = cmiManagerFactory;
        this.mapDialogFactory = mapDialogFactory;
        this.hltf = hltf;
        this.tasksFactory = tasksFactory;
        this.taskService = taskService;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.bonfireLicenseUtil = bonfireLicenseUtil;
    }

    public AdaptivePropertiesDialog createAdaptivePropertiesDialog(ModelRspecEditor modelRspecEditor, FXRspecNode node) {
        return new AdaptivePropertiesDialog(modelRspecEditor, node, config, authorityList, cmiManagerFactory, mapDialogFactory,
                hltf, tasksFactory, taskService, aggregateManagerWrapperFactory, bonfireLicenseUtil);
    }

    public void showAdaptivePropertiesDialog(Window parent,
                                             ModelRspecEditor modelRspecEditor, FXRspecNode node) {
        Scene scene = new Scene(createAdaptivePropertiesDialog(modelRspecEditor, node));

        Stage dialogStage = new Stage();
        StageUtils.setJFedStageIcons(dialogStage);
        dialogStage.setScene(scene);

        dialogStage.setTitle("Properties of " + node.getClientId());
        dialogStage.setMinHeight(500);
        dialogStage.setMaxHeight(600);
        dialogStage.setMinWidth(500);
        FXUtils.centerToOwner(parent, dialogStage);

        dialogStage.showAndWait();
    }
}
