package be.iminds.ilabt.jfed.experimenter_gui.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.dialogs.SelectAuthoritiesDialog;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ExperimentEditorTabFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewControllerFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tutorials.TutorialViewsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ExperimentEditorTabHost;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.SfaModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.collections.ObservableMap;
import javafx.stage.Window;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

/**
 * User: twalcari
 * Date: 12/1/2014
 * Time: 17:40
 */
@Singleton
public class TasksFactoryImpl implements TasksFactory {
    private final SfaModel sfaModel;
    private final HighLevelTaskFactory hltf;
    private final ExperimentViewControllerFactory experimentViewControllerFactory;
    private final ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers;
    private final Provider<SelectAuthoritiesDialog> selectAuthoritiesDialogProvider;
    private final Provider<TutorialViewsFactory> tutorialViewsFactoryProvider;
    private final ExperimentEditorTabFactory experimentEditorTabFactory;
    private final ExperimentEditorTabHost experimentEditorTabHost;
    private final AuthorityListModel authorityListModel;
    private final AuthorityFinder authorityFinder;
    private final JFedPreferences jFedPreferences;

    @Inject
    TasksFactoryImpl(SfaModel sfaModel, TaskThread tt, HighLevelTaskFactory hltf,
                     Provider<TutorialViewsFactory> tutorialViewsFactoryProvider, AuthorityListModel authorityListModel,
                     AuthorityFinder authorityFinder,
                     JFedPreferences jFedPreferences,
                     ExperimentViewControllerFactory experimentViewControllerFactory,
                     ObservableMap<GeniUrn, ExperimentViewController> experimentViewControllers,
                     Provider<SelectAuthoritiesDialog> selectAuthoritiesDialogProvider,
                     ExperimentEditorTabFactory experimentEditorTabFactory,
                     ExperimentEditorTabHost experimentEditorTabHost) {
        this.sfaModel = sfaModel;
        this.hltf = hltf;
        this.tutorialViewsFactoryProvider = tutorialViewsFactoryProvider;
        this.authorityListModel = authorityListModel;
        this.authorityFinder = authorityFinder;
        this.jFedPreferences = jFedPreferences;
        this.experimentViewControllerFactory = experimentViewControllerFactory;
        this.experimentViewControllers = experimentViewControllers;
        this.selectAuthoritiesDialogProvider = selectAuthoritiesDialogProvider;
        this.experimentEditorTabFactory = experimentEditorTabFactory;
        this.experimentEditorTabHost = experimentEditorTabHost;
    }

    @Override
    public RecoverSliceTask createRecoverSliceTask(GeniUrn sliceUrn) {
        return new RecoverSliceTask(sliceUrn, sfaModel, hltf, authorityListModel, authorityFinder, jFedPreferences, experimentViewControllerFactory, experimentViewControllers, selectAuthoritiesDialogProvider);

    }

    @Override
    public RecoverSliceTask createRecoverSliceTask(Slice slice) {
        return new RecoverSliceTask(slice, sfaModel, hltf, authorityListModel, authorityFinder, jFedPreferences, experimentViewControllerFactory, experimentViewControllers, selectAuthoritiesDialogProvider);
    }

    @Override
    public DeleteDiskImageTask createDeleteDiskImageTask(SfaAuthority auth, AggregateManagerWrapper.ImageInfo imageInfo) {
        return new DeleteDiskImageTask(auth, imageInfo, hltf);
    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, File file) throws FileNotFoundException{
        return new OpenRequestRspecTask(parentWindow, file, experimentEditorTabFactory, experimentEditorTabHost);

    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, InputStream inputStream, long fileLength, String name, File defaultSaveFile){
        return new OpenRequestRspecTask(parentWindow,inputStream, fileLength,name,defaultSaveFile,experimentEditorTabFactory,experimentEditorTabHost);
    }

    @Override
    public OpenRequestRspecTask createOpenRequestRspecTask(Window parentWindow, String urlString) throws IOException {
        return new OpenRequestRspecTask(parentWindow, urlString, experimentEditorTabFactory, experimentEditorTabHost);
    }

    @Override
    public FetchAllSlicesInfoTask createFetchAllSlicesInfoTask() {
        return new FetchAllSlicesInfoTask(sfaModel, hltf);
    }


    @Override
    public OpenTutorialTask createOpenTutorialTask(Window dialogOwnerWindow, String urlString) throws IOException{
        return new OpenTutorialTask(dialogOwnerWindow, urlString, tutorialViewsFactoryProvider.get());
    }
}
