package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.highlevel.jobs.report.JobReportWrappingLogger;
import be.iminds.ilabt.jfed.lowlevel.api.AbstractGeniAggregateManager;
import be.iminds.ilabt.jfed.lowlevel.api.StitchingComputationService;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.stitching.StitchingDirector;

import javax.annotation.Nullable;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * User: twalcari
 * Date: 1/27/2016
 * Time: 11:24
 */
class StitchingAllocationDirector implements AllocationDirector {
    private final Experiment experiment;
    private final StitchingDirector director;

    StitchingAllocationDirector(Experiment experiment, AuthorityListModel authorityListModel,
                                AuthorityFinder authorityFinder,
                                JobReportWrappingLogger stitchingDirectorLog) {
        this.experiment = experiment;
        this.director = new StitchingDirector(authorityListModel, authorityFinder,
                Date.from(experiment.getRequestedEndTime()), stitchingDirectorLog);
    }

    @Override
    public void reportGiveUp() {
        synchronized (director) {
            director.reportGiveup();
        }
    }

    @Override
    public boolean isGaveUp() {
        synchronized (director) {
            return director.isGaveUp();
        }
    }

    @Override
    public boolean areAnyActionsLeft() {
        synchronized (director) {
            return director.areAnyActionsLeft();
        }
    }

    @Override
    public List<StitchingDirector.StitchAction> getStitchActions() {
        synchronized (director) {
            return director.getStitchActions();
        }
    }

    @Override
    public StitchingDirector.AllocateVerdict processAllocateResult(StitchingDirector.StitchAction stitchAction, AbstractGeniAggregateManager.AggregateManagerReply reply) {
        synchronized (director) {
            return director.processAllocateResult(stitchAction, reply);
        }
    }

    @Override
    public void processListResourcesResult(SfaAuthority auth, @Nullable String advertisementRspec) {
        synchronized (director) {
            director.processListResourcesResult(auth, advertisementRspec);
        }
    }

    @Override
    public void processDeleteResult(SfaAuthority auth) {
        synchronized (director) {
            director.processDeleteResult(auth);
        }
    }

    @Override
    public StitchingDirector.AllocateVerdict reportReadyForRetry(SfaAuthority auth) {
        synchronized (director) {
            return director.reportReadyForRetry(auth);
        }
    }

    public void setComputePathResult(StitchingComputationService.ComputePathResult computePathResult) {
        director.setComputePathResult(computePathResult);
    }

    public Collection<SfaAuthority> getAllAuthorities() {
        return director.getAllAuthorities();
    }
}
