package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.util.GeniUrn;

import java.util.Collections;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 16:38
 */
public class ReloadOSTask extends Task {
    private final Slice slice;
    private final SfaAuthority auth;
    private final GeniUrn reloadOSTarget;

    private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    private final HighLevelTaskFactory highLevelTaskFactory;

    public ReloadOSTask(Slice slice, SfaAuthority auth, GeniUrn reloadOSTarget,
                        AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                        HighLevelTaskFactory highLevelTaskFactory) {
        super("Reload OS on " + reloadOSTarget);
        this.slice = slice;
        this.auth = auth;
        this.reloadOSTarget = reloadOSTarget;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;
    }

    @Override
    public void doTask(TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);
        if (!amWrapper.isReloadOSSupported())
            throw new JFedHighLevelException("Authority " + auth.getUrnString() + " does not support reloadOS.");
        amWrapper.reloadOS(reloadOSTarget, slice.getCredentials());
    }

    @Override
    public List<Task> initDependsOn() {
        return Collections.singletonList(highLevelTaskFactory.getSliceCredential(slice));
    }
}
