package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.lowlevel.api.ReputationService;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.connection.HttpConnection;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JavaFXLogger;

/**
 * User: twalcari
 * Date: 10/29/2015
 * Time: 17:07
 */
public abstract class ReputationTask extends Task {
    private static final String REPUTATION_URN = "urn:publicid:IDN+reputation+authority+cm";

    private final JavaFXLogger logger;
    private final JFedPreferences jFedPreferences;
    private final AuthorityList authorityList;
    private final JFedConnectionProvider connectionProvider;

    protected final GeniUserProvider geniUserProvider;

    public ReputationTask(String name, JavaFXLogger logger,
                          JFedPreferences jFedPreferences, AuthorityList authorityList,
                          JFedConnectionProvider connectionProvider, GeniUserProvider geniUserProvider) {
        super(name);
        this.logger = logger;
        this.jFedPreferences = jFedPreferences;
        this.authorityList = authorityList;
        this.connectionProvider = connectionProvider;
        this.geniUserProvider = geniUserProvider;
    }


    protected HttpConnection createConnection() throws JFedException {
        final AuthorityInfo ai = authorityList.getByUrnExact(REPUTATION_URN);
        if (ai == null)
            throw new JFedHighLevelException("No reputation authority defined! Check your testbeds.xml");
        SfaAuthority slaAuth = ai.getSfaAuthority();


        final GeniUser loggedInGeniUser = geniUserProvider.getLoggedInGeniUser();
        return (HttpConnection) connectionProvider.getConnectionByAuthority(loggedInGeniUser, slaAuth, new ServerType(ServerType.GeniServerRole.REPUTATION_SERVICE, 1));
    }

    @Override
    protected final void doTask(TaskExecution taskExecution) throws JFedException, InterruptedException {
        final ReputationService reputationService = new ReputationService(JavaFXLogger.wrap(logger, taskExecution), jFedPreferences);
        doReputationTask(taskExecution, reputationService);

    }

    protected abstract void doReputationTask(TaskExecution taskExecution, ReputationService reputationService) throws JFedException, InterruptedException;

}
