package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.AnyCredential;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.UserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.util.JavaFXLogger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 6/16/2015
 * Time: 16:25
 */
public class UserCredentialTask extends Task {
    private static final Logger LOG = LoggerFactory.getLogger(UserCredentialTask.class);

    private final UserAndSliceApiWrapper w;
    private final JavaFXLogger logger;

    public UserCredentialTask(UserAndSliceApiWrapper w, JavaFXLogger logger) {
        super("Get User Credential");
        this.w = w;
        this.logger = logger;
    }

    @Override
    public void doTask(TaskExecution taskExecution) throws JFedException, InterruptedException {
        LOG.debug("Get User Credential started");

        List<AnyCredential> res =
                w.getLoggedinUserCredentials(JavaFXLogger.wrap(logger, taskExecution));

        try {
            List<Date> expireDatesRecv =
                    res.stream().map(AnyCredential::getExpiresDate).collect(Collectors.toList());
            List<Date> expireDatesStored =
                    w.getCachedUserCredentialsForAM().stream().map(AnyCredential::getExpiresDate).collect(Collectors.toList());
            boolean hasUserCredentials = w.hasUserCredentials();

            LOG.debug("GetCredential call finished. Expire dates of received creds: " + expireDatesRecv +
                    ". Expire dates of stored credentials: " + expireDatesStored +
                    "  hasUserCredentials=" + hasUserCredentials);

        } catch (Throwable e) {
            LOG.warn("Exception while creating log output", e);
        }

        assert res != null;
        assert !res.isEmpty();
    }

    @Override
    public boolean newExecutionNeeded() {
        boolean hasUserCredentials = w.hasUserCredentials();
        try {
            List<Date> expireDates =
                    w.getCachedUserCredentialsForAM() != null ?
                            w.getCachedUserCredentialsForAM().stream()
                                    .map(AnyCredential::getExpiresDate)
                                    .collect(Collectors.toList())
                            : null;
            LOG.info("getUserCredential newExecutionNeeded() called -> hasUserCredentials={} now={} expireDates={}",
                    hasUserCredentials, new Date(), expireDates);

        } catch (Throwable t) {
            LOG.warn("Exception in logging ignored. hasUserCredentials=" + hasUserCredentials, t);
        }
        return !hasUserCredentials;
    }

    @Override
    public List<Task> initDependsOn() {
        return Collections.emptyList();
    }
}
