package be.iminds.ilabt.jfed.lowlevel.ssh_key_info;

import be.iminds.ilabt.jfed.util.tmp_file_helpers.TmpFile;

import javax.annotation.Nullable;
import java.security.PrivateKey;
import java.security.PublicKey;

/**
 * Ssh Key info. This offers access to various info needed for SSH.
 * Sometimes, this might require making temporary files.
 *
 * Call release() to delete any temporary files containing unencrypted private keys
 * */
public interface SshKeyInfo {
    /** file containing private key. may be encrypted. Non putty format. never null. Will also make sure a .pub file with the public key is in the same dir. */
    TmpFile getPrivateKeyFile();
    /** file containing private key. may be encrypted. Putty format. never null (on windows). */
    TmpFile getPuttyKeyFile();

    /** file containing private key. Non putty format. never null. Will also make sure a .pub file with the public key is in the same dir. */
    TmpFile getUnencryptedPrivateKeyFile();
    /** file containing private key. Putty format. */
    @Nullable
    TmpFile getUnlockedPuttyKeyFile();

    /** Does this SSH key info have access to the unencrypted private key? */
    boolean hasUnencryptedPrivateKey();

    /** may be null */
    @Nullable
    PublicKey getPublicKey();
    /** may be null */
    @Nullable
    PrivateKey getPrivateKey();
}